design <- function (Materials, preparation, VolumeModel, densities, 
LowerBoundsVolume, UpperBoundsVolume, LowerBoundsMass, 
UpperBoundsMass, limdesign, accuracy) {

#Materials: List of selected Materials
#preparation: data frame contains column for diameters and columns 
#             for Materials as cumulative passthrough curves (CPFT)
#VolumeModel: vector of defined CPFT-model in vol% with values for 
#             every diameter 
#densities: vector contains densities of Materials
#limdesign, accuracy: limit of iterations and accuracy (decimal 
#                     places) for calculations defined by the user 
#                     in the settings
#LowerBoundsVolume, UpperBoundsVolume: Specified vol%/100% bounds, 
#                                      if not specified 0 for lower 
#                                      bounds and 1 for upper ones
#LowerBoundsMass, UpperBoundsMass: Specified mass%/100% bounds, if 
#                                  not specified 0 for lower bounds 
#                                  and 1 for upper bounds

 preparation <- cbind(preparation, VolumeModel)#add vol% model to 
                                               #dataframe as column

 run <- 0 #iteration step of normalization to 100 vol%
 runx <- 0 #iteration step of mass% calculation

 mcheck <- 0 #evaluation variable for mass% calculation

 while (mcheck!=1) { #mass% calculation-iteration

  if (run+runx > limdesign) { #limit reached
   #return to: model, bounds, material selection or main menu
  }

  check <- 0 #evaluation variable for normalization to 100 vol%
 
  while (check!=1) { #normalization to 100vol%-iteration

   if (run+runx > limdesign) { #limit reached
    #return to: model, bounds, material selection or main menu
   }
  
   fit <- nls(VolumeModel~Coefficients*Materials, data=preparation, 
   start=WithinBounds, algorithm="port", lower=LowerBoundsVolume, 
   upper=UpperBoundsVolume)#returns coefficients equivalent to vol% 
                           #of the Materials in the batch
 
   if (is.null(fit)) { #nls()-error
    #return to: model, bounds, material selection or main menu
   }

   check = sum(coef(fit)) #Auxiliary condition, that coefficients 
                          #(vol%) add up to 1 (100%) -> check if 
                          #it does is done in the condition of the 
                          #corresponding while-loop

   preparation <- rbind(preparation, c(100,...))#adds a row of 100s 
                                                #to preparation, 
                                                #giving more focus 
                                                #in the next step
                                                #of the iteration  
                                                #on the 100vol%
  
   run=run+1
   
  } #end while loop of normalization to 100vol%-iteration
 
  Den <- 0 #Calculate Batch density
  for (n in seq(from=1, to=length(Materials), by=1)) { 
  Den <- Den+coef(fit)[n]*densities[n] }

  masses <- vector(length=length(Materials)) #calculate mass% of 
                                             #Materials in batch 
  for (o in seq(from=1, to=length(Materials), by=1)) { 
   masses[o] <- coef(fit)[o]*densities[o]/Den 
  }

  mcheck <- 1
  for (p in seq(from=1, to=length(Materials), by=1)) {#check if 
                                                      #mass% are
                                                      #within set 
                                                      #mass%-
                                                      #bounds. If
                                                      #too low, 
                                                      #increase 
                                                      #by arbitrary
                                                      #amount
                                                      #1/10^();
                                                      #if too high, 
                                                      #decrease by 
                                                      #this amount
   if (masses[p] < LowerBoundsMass[p]) { 
    LowerBoundsVolume[p] = LowerBoundsVolume[p] + 1/10^(accuracy+2) 
    mcheck=mcheck+1 
   } else if (masses[p] > UpperBoundsMass) {
    UpperBoundsVolume[p] = UpperBoundsVolume[p] - 1/10^(accuracy+2) 
    mcheck=mcheck+1
   }
  }

  for (r in seq(from=1, to=length(Materials), by=1)) { #move bounds  
                                                       #if they 
                                                       #catch up 
                                                       #with each
                                                       #other
   if (LowerBoundsVolume[r] > UpperBoundsVolume[r]) {
    if (masses[r] < LowerBoundsMass[r]) {
     UpperBoundsVolume[r] <- LowerBoundsVolume[r] 
    } else if (masses[r] > UpperBoundsMass[r]) { 
     LowerBoundsVolume[r] <- UpperBoundsVolume[r] 
    }
   }
  }

  runx=runx+1

 } #end while-loop of mass% calculation-iteration

#show optimization results: 
#coef(fit) contains vol% for Materials
#masses contains mass% for Materials

} #end design-function








