calcparams <- function (Materials, preparation, densities, Model, 
BoundsMass, BoundsVolume, LowerBoundsSizes, UpperBoundsSizes,Batch,
LowerBoundsModuli, UpperBoundsModuli, limparcalc, accuracy, Den) {
  
#Materials: List of selected Materials
#preparation: data frame contains column for Diameters and columns 
#             for Materials as cumulative passthrough curves (CPFT)
#Model: Chosen model
#densities: vector contains densities of Materials
#limparcalc, accuracy: limit of iterations and accuracy (decimal 
#                     places) for calculations defined by the user 
#                     in the settings
#BoundsVolume, BoundsMass: Defined batch
#LowerBoundsSizes, UpperBoundsSizes: Specified bounds for the 
#                                    particle size parameters of
#                                    the chosen Model
#LowerBoundsModuli, UpperBoundsModuli: Specified bounds for the
#                                      distribution moduli of the
#                                      chosen Model
#Den: batch density
#Batch: Cumulative sum curve of batch 

 nacount <<- 0 #counter for failed nls-fittings
  
 preparation <- cbind(preparation, Batch) #add batch to dataframe

 run <- 0 #Iteration step of particle size calculation
 step <- 0 #Refining step of single iteration step

 if (Model == "Andreasen model") {

  dmaxl <<- LowerBoundsSizes   #Initialize running variables 
  dmaxu <<- UpperBoundsSizes   #required for fitting particle size 
  dmaxmin <<- LowerBoundsSizes #dmax

  while (TRUE) { #while loop to find optimal dmax

   if (run > limparcalc) { #limit reached
    #return to: model, batch def., material selection or main menu
   }

   run <- run + 1
   step <- 1

   if (dmaxl == dmaxu) { #Calculate final distribution moduli when
                         #fitting of particle size dmax finished

    dmax <- dmaxmin #Initialze dmax with optimum value

    #prepare sub-dataset which ends at dmax for fitting:
    modelsub <- subset(preparation, preparation$Diameter <= dmax)

    fit <- nls(Batch ~ 100*(Diameter/dmax)^Vn , data=modelsub, 
    start=WithinBounds, algorithm="port", lower=LowerBoundsModuli, 
    upper=UpperBoundsModuli) #returns distribution modulus (Vn) 
                             #stored in coef(fit)
  
    if (is.null(fit)) { #nls()-error
     #error mesage window
     #return to: model, bounds, material selection or main menu
    }
   
    #Calculate VolumeModel for chosen Model with fitted value
    #saved in coef(fit)
    for (i in seq(from=1,to=length(Materials),by=1)) {
     if (dmax < preparation$Diameter[i]) { VolumeModel[i] <<- 100 }
     else { VolumeModel[i]<<-100*(preparation$Diameter[i]/
                                            dmax)^coef(fit)[1] }
    }
    
    break #after dmax iterated and distribution modulus fitted,
          #while loop is left

   } else { #dmaxl != dmaxu, fitting of dmax not finished

    #Initialize logarithmic steps for dmax within bounds of last
    #run <-> dmaxl and dmaxu
    dmax <- vector(length=4)
    dmax[1] <- dmaxl
    dmax[2] <- 10^(log10(dmaxl)+(log10(dmaxu)-log10(dmaxl))/3)
    dmax[3] <- 10^(log10(dmaxu)-(log10(dmaxu)-log10(dmaxl))/3)
    dmax[4] <- dmaxu

    sumerrsq <- rep(0, length(dmax)) #Evaluation variable for dmax:
                                     #sum of squared deviations
    
    #For every dmax of the dmax-steps, a distirbution modulus to
    #give optimal results is fitted. Then, the calculable model 
    #and given batch are compared and the deviations characterized
    #by the sum of squared deviations per component size are
    #calculated for every dmax of the dmax-steps:
    for(j in seq(from=1, to=length(dmax), by=1)) {
     step <- j

     modelsub<-subset(preparation, preparation$Diameter <= dmax[j])

     fit <- nls(Batch ~ 100*(Diameter/dmax[j])^Vn , data=modelsub, 
     start=WithinBounds, algorithm="port", lower=LowerBoundsModuli, 
     upper=UpperBoundsModuli) #optimal dist. modulus (Vn) for 
                              #specific dmax-step

     if (is.null(fit)) { #nls()-error
      sumerrsq[j] <- NA
      nacount <<- nacount+1
      next #going on with next dmax-step
     }
      
     #create VolumeModel-column and calculate sum of squared dev.:
     errsq <- vector(length=length(preparation$Diameter))
     for (i in seq(from=1,to=length(preparation$Diameter),by=1)) {
      if (dmax[j]<preparation$Diameter[i]) {VolumeModel[i] <<- 100}
      else { VolumeModel[i] <<- 100*(preparation$Diameter[i]/
                                            dmax[j])^coef(fit)[1] }
            
      errsq[i] <- ((VolumeModel[i]-Batch[i])* #squared deviation of
               (VolumeModel[i]-Batch[i]))     #batch and model per
                                              #component size
      sumerrsq[j]<- sumerrsq[j]+errsq[i] #sum of squared deviations
     }
    
    } #end for j in dmax loop

    if (sum(is.na(sumerrsq)) == 4) { #true if nls()-errors occurred
                                     #for all dmax-steps
     #return to: model, bounds, material selection or main menu
    }

    #find the dmax with the minimum sum of squared dev. for the
    #curve fit and set neighbouring iteration steps as new bounds
    if (which.min(sumerrsq) == 1) {
     dmaxl <<- dmax[1]
     dmaxu <<- dmax[2]
    } else if (which.min(sumerrsq) == 2) {
     dmaxl <<- dmax[1]
     dmaxu <<- dmax[3]
    } else if (which.min(sumerrsq) == 3) {
     dmaxl <<- dmax[2]
     dmaxu <<- dmax[4]
    } else {
     dmaxl <<- dmax[3]
     dmaxu <<- dmax[4]
    }

    #save dmax that returned minimum sum of squared deviations
    dmaxmin <<- dmax[which.min(sumerrsq)] 

   }#end else (dmaxl != dmaxu)

  }#end while loop Andreasen

  optd <- dmax      #save optimal parameters
  optn <- coef(fit)

 } #end if (Model == "Andreasen model")

 if (nacount > 0) { #did any nls-fittings fail?
  #message, that there were failed nls-fittings although a result
  #was found
 }

 #show optimization results: 
 #optd contains optimal values for the particle size parameters
 #optn contains optimal values for distribution moduli

} #end of calcparams (calculate model parameters) function
